@@ -1,4 +1,5 @@
 """System static utilities being used by the modules."""
+
 import base64
 import os
 import platform
@@ -7,7 +8,7 @@ import stat
 import subprocess
 import sys
 import sqlite3
-from six.moves import input
+from typing import Optional
 
 from . import constants
 
@@ -15,12 +16,14 @@ from . import constants
 # Flag that controls how user confirmation works.
 # If True, the user wants to say "yes" to everything.
 FORCE_YES = False
+# If True, the user wants to say "no" to everything.
+FORCE_NO = False
 
 # Flag that control if mackup can be run as root
 CAN_RUN_AS_ROOT = False
 
 
-def confirm(question):
+def confirm(question: str) -> bool:
     """
     Ask the user if he really wants something to happen.
 
@@ -32,6 +35,8 @@ def confirm(question):
     """
     if FORCE_YES:
         return True
+    if FORCE_NO:
+        return False
 
     while True:
         answer = input(question + " <Yes|No> ").lower()
@@ -46,7 +51,7 @@ def confirm(question):
     return confirmed
 
 
-def delete(filepath):
+def delete(filepath: str) -> None:
     """
     Delete the given file, directory or link.
 
@@ -68,7 +73,7 @@ def delete(filepath):
         shutil.rmtree(filepath)
 
 
-def copy(src, dst):
+def copy(src: str, dst: str) -> None:
     """
     Copy a file or a folder (recursively) from src to dst.
 
@@ -102,7 +107,7 @@ def copy(src, dst):
 
     # We need to copy a whole folder
     elif os.path.isdir(src):
-        shutil.copytree(src, dst)
+        shutil.copytree(src, dst, dirs_exist_ok=True)
 
     # What the heck is this?
     else:
@@ -112,7 +117,7 @@ def copy(src, dst):
     chmod(dst)
 
 
-def link(target, link_to):
+def link(target: str, link_to: str) -> None:
     """
     Create a link to a target file or a folder.
 
@@ -145,7 +150,7 @@ def link(target, link_to):
     os.symlink(target, link_to)
 
 
-def chmod(target):
+def chmod(target: str) -> None:
     """
     Recursively set the chmod for files to 0600 and 0700 for folders.
 
@@ -181,7 +186,7 @@ def chmod(target):
         raise ValueError("Unsupported file type: {}".format(target))
 
 
-def error(message):
+def error(message: str) -> None:
     """
     Throw an error with the given message and immediately quit.
 
@@ -193,7 +198,7 @@ def error(message):
     sys.exit(fail + "Error: {}".format(message) + end)
 
 
-def get_dropbox_folder_location():
+def get_dropbox_folder_location() -> str:
     """
     Try to locate the Dropbox folder.
 
@@ -211,7 +216,7 @@ def get_dropbox_folder_location():
     return dropbox_home
 
 
-def get_google_drive_folder_location():
+def get_google_drive_folder_location() -> str:
     """
     Try to locate the Google Drive folder.
 
@@ -226,7 +231,7 @@ def get_google_drive_folder_location():
     if os.path.isfile(yosemite_gdrive_db):
         gdrive_db_path = yosemite_gdrive_db
 
-    googledrive_home = None
+    googledrive_home: Optional[str] = None
 
     gdrive_db = os.path.join(os.environ["HOME"], gdrive_db_path)
     if os.path.isfile(gdrive_db):
@@ -253,7 +258,7 @@ def get_google_drive_folder_location():
     return googledrive_home
 
 
-def get_icloud_folder_location():
+def get_icloud_folder_location() -> str:
     """
     Try to locate the iCloud Drive folder.
 
@@ -270,7 +275,7 @@ def get_icloud_folder_location():
     return str(icloud_home)
 
 
-def is_process_running(process_name):
+def is_process_running(process_name: str) -> bool:
     """
     Check if a process with the given name is running.
 
@@ -291,7 +296,7 @@ def is_process_running(process_name):
     return is_running
 
 
-def remove_acl(path):
+def remove_acl(path: str) -> None:
     """
     Remove the ACL of the file or folder located on the given path.
 
@@ -311,7 +316,7 @@ def remove_acl(path):
         subprocess.call(["/bin/setfacl", "-R", "-b", path])
 
 
-def remove_immutable_attribute(path):
+def remove_immutable_attribute(path: str) -> None:
     """
     Remove the immutable attribute of the given path.
 
@@ -334,7 +339,7 @@ def remove_immutable_attribute(path):
         subprocess.call(["/usr/bin/chattr", "-R", "-f", "-i", path])
 
 
-def can_file_be_synced_on_current_platform(path):
+def can_file_be_synced_on_current_platform(path: str) -> bool:
     """
     Check if the given path can be synced locally.
 
